/**
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */
package com.perforce.maven.scm.provider.p4.command.info;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.scm.CommandParameter;
import org.apache.maven.scm.CommandParameters;
import org.apache.maven.scm.ScmException;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.ScmVersion;
import org.apache.maven.scm.command.AbstractCommand;
import org.apache.maven.scm.command.info.InfoItem;
import org.apache.maven.scm.command.info.InfoScmResult;
import org.apache.maven.scm.provider.ScmProviderRepository;

import com.perforce.maven.scm.provider.p4.command.P4Command;
import com.perforce.maven.scm.provider.p4.util.P4Utils;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.IChangelistSummary;

/**
 * SCM Provider Info command implementation
 */
public class P4InfoCommand
    extends AbstractCommand
    implements P4Command
{

    /** The Constant command. */
    public static final String command = "Perforce info command";

    private IClient client;

    public void setClient( IClient client )
    {
        this.client = client;
    }

    protected InfoScmResult executeCommand( ScmProviderRepository repo, ScmFileSet files, CommandParameters params )
        throws ScmException
    {
        String datePattern = "yyyy/MM/dd HH:mm:ss";
        if ( params != null )
        {
            datePattern = params.getString( CommandParameter.CHANGELOG_DATE_PATTERN, datePattern );
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat( datePattern );

        List<InfoItem> infoItems = new ArrayList<InfoItem>();
        InfoItem infoItem = new InfoItem();
        ScmVersion scmVersion = null;
        if ( params != null )
        {
            scmVersion = params.getScmVersion( CommandParameter.SCM_VERSION, null );
        }

        // get the latest revision at the server
        {
            IChangelistSummary cl = P4Utils.getLatestChangelist( client, files.getBasedir(), scmVersion, true );
            infoItem.setRevision( Long.toString( cl.getId() ) );
            infoItem.setLastChangedRevision( Long.toString( cl.getId() ) );
            infoItem.setLastChangedAuthor( cl.getUsername() );
            infoItem.setLastChangedDate( dateFormat.format( cl.getDate() ) );
        }

        // get the revision from last sync
        {
            IChangelistSummary cl = P4Utils.getLatestChangelist( client, files.getBasedir(), scmVersion, false );
            if ( cl != null )
            {
                infoItem.setLastChangedRevision( Long.toString( cl.getId() ) );
                infoItem.setLastChangedAuthor( cl.getUsername() );
                infoItem.setLastChangedDate( dateFormat.format( cl.getDate() ) );
            }
            else
            {
                // last revision is not available due to client is not sync yet
                // so last revision is the same as latest from repo
            }
        }

        infoItems.add( infoItem );

        InfoScmResult result = new InfoScmResult( command, infoItems );

        return result;
    }
}
