/**
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */
package com.perforce.maven.scm.provider.p4.command.remove;

import java.util.List;

import org.apache.maven.scm.ScmException;
import org.apache.maven.scm.ScmFile;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.ScmResult;
import org.apache.maven.scm.command.remove.AbstractRemoveCommand;
import org.apache.maven.scm.command.remove.RemoveScmResult;
import org.apache.maven.scm.provider.ScmProviderRepository;

import com.perforce.maven.scm.provider.p4.command.P4Command;
import com.perforce.maven.scm.provider.p4.util.P4Utils;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.option.client.DeleteFilesOptions;

/**
 * The class uses the Perforce delete command to open existing files for deletion from the depot.
 */
public class P4RemoveCommand
    extends AbstractRemoveCommand
    implements P4Command
{

    /** The Constant command. */
    public static final String command = "Perforce delete command";

    private IClient client;

    public void setClient( IClient client )
    {
        this.client = client;
    }

    /**
     * @see org.apache.maven.scm.command.remove.AbstractRemoveCommand
     *      executeRemoveCommand(org.apache.maven.scm.provider.ScmProviderRepository, org.apache.maven.scm.ScmFileSet,
     *      java.lang.String)
     */
    protected ScmResult executeRemoveCommand( ScmProviderRepository repo, ScmFileSet files, String message )
        throws ScmException
    {

        P4RemoveResult p4Result = new P4RemoveResult();

        List<IFileSpec> fileSpecs = P4Utils.scmFileSet2P4FileSpecsWithNoP4WildCard( files );

        try
        {
            if ( !fileSpecs.isEmpty() )
            {
                DeleteFilesOptions options = new DeleteFilesOptions();

                List<IFileSpec> retFileSpecs = client.deleteFiles( fileSpecs, options );

                p4Result.process( retFileSpecs );
                p4Result.processRemovals( retFileSpecs );
            }

        }
        catch ( P4JavaException e )
        {
            throw new ScmException( e.getLocalizedMessage(), e );
        }

        List<ScmFile> removals = p4Result.getRemovals();
        return new RemoveScmResult( command, removals );
    }

}
